package gregapi.block.multitileentity;

import static gregapi.data.CS.*;
import gregapi.block.multitileentity.IMultiTileEntity.IMTE_AddToolTips;
import gregapi.block.multitileentity.IMultiTileEntity.IMTE_CanPlace;
import gregapi.block.multitileentity.IMultiTileEntity.IMTE_GetExplosionResistance;
import gregapi.block.multitileentity.IMultiTileEntity.IMTE_GetItemName;
import gregapi.block.multitileentity.IMultiTileEntity.IMTE_GetLifeSpan;
import gregapi.block.multitileentity.IMultiTileEntity.IMTE_GetSubItems;
import gregapi.block.multitileentity.IMultiTileEntity.IMTE_HasMultiBlockMachineRelevantData;
import gregapi.block.multitileentity.IMultiTileEntity.IMTE_OnCrafted;
import gregapi.block.multitileentity.IMultiTileEntity.IMTE_OnDespawn;
import gregapi.block.multitileentity.IMultiTileEntity.IMTE_OnPlaced;
import gregapi.code.ArrayListNoNulls;
import gregapi.cover.CoverData;
import gregapi.cover.ITileEntityCoverable;
import gregapi.data.LH;
import gregapi.data.MT;
import gregapi.item.CreativeTab;
import gregapi.item.IItemGT;
import gregapi.item.IItemNoGTOverride;
import gregapi.item.IItemUpdatable;
import gregapi.oredict.IOreDictItemDataOverrideItem;
import gregapi.oredict.OreDictItemData;
import gregapi.tileentity.ITileEntity;
import gregapi.tileentity.ITileEntityFoamable;
import gregapi.tileentity.ITileEntityMachineBlockUpdateable;
import gregapi.util.OM;
import gregapi.util.UT;

import java.util.List;

import net.minecraft.block.Block;
import net.minecraft.block.BlockSnow;
import net.minecraft.client.renderer.texture.IIconRegister;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.entity.item.EntityItem;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.item.Item;
import net.minecraft.item.ItemBlock;
import net.minecraft.item.ItemStack;
import net.minecraft.util.ChunkCoordinates;
import net.minecraft.world.World;
import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;

/**
 * @author Gregorius Techneticies
 */
public class MultiTileEntityItemInternal extends ItemBlock implements IItemUpdatable, IOreDictItemDataOverrideItem, IItemGT, IItemNoGTOverride {
	public final MultiTileEntityBlockInternal mBlock;
	
	public MultiTileEntityItemInternal(Block aBlock) {
		super(aBlock);
        setMaxDamage(0);
        setHasSubtypes(true);
		mBlock = (MultiTileEntityBlockInternal)aBlock;
	}
	
	@Override
	public String getItemStackDisplayName(ItemStack aStack) {
		MultiTileEntityContainer tTileEntityContainer = mBlock.mMultiTileEntityRegistry.getNewTileEntityContainer(aStack);
		if (tTileEntityContainer != null && tTileEntityContainer.mTileEntity instanceof IMTE_GetItemName) return ((IMTE_GetItemName)tTileEntityContainer.mTileEntity).getItemName(aStack, super.getItemStackDisplayName(aStack));
		return super.getItemStackDisplayName(aStack);
	}
	
	@Override
	public void addInformation(ItemStack aStack, EntityPlayer aPlayer, List aList, boolean aF3_H) {
		MultiTileEntityContainer tTileEntityContainer = mBlock.mMultiTileEntityRegistry.getNewTileEntityContainer(aStack);
		if (tTileEntityContainer == null) {aList.add("INVALID ITEM! THIS IS A BUG IF ACQUIRED IN A LEGIT WAY!"); return;}
		if (tTileEntityContainer.mTileEntity instanceof IMTE_AddToolTips) try {((IMTE_AddToolTips)tTileEntityContainer.mTileEntity).addToolTips(aList, aStack, aF3_H);} catch(Throwable e) {e.printStackTrace(ERR);}
		if (tTileEntityContainer.mTileEntity instanceof IMTE_GetExplosionResistance) {
			double tResistance = ((IMTE_GetExplosionResistance)tTileEntityContainer.mTileEntity).getExplosionResistance();
			aList.add(LH.Chat.WHITE + LH.get(LH.TOOLTIP_BLASTRESISTANCE) + LH.Chat.ORANGE + (tResistance>0?tResistance:"None"));
		}
		aList.add(LH.Chat.DGRAY + LH.get(LH.TOOL_TO_HARVEST) + ": " + LH.Chat.WHITE + LH.get(TOOL_LOCALISER_PREFIX + tTileEntityContainer.mBlock.getHarvestTool(tTileEntityContainer.mBlockMetaData), "Unknown") + " (" + tTileEntityContainer.mBlock.getHarvestLevel(tTileEntityContainer.mBlockMetaData) + ")");
		// Remove all Nulls and fix eventual Formatting mistakes.
		for (int i = 0, j = aList.size(); i < j; i++) if (aList.get(i) == null) {aList.remove(i--); j--;} else aList.set(i, LH.Chat.GRAY + aList.get(i) + LH.Chat.RESET);
    }
	
	public int onDespawn(EntityItem aEntity, ItemStack aStack) {
		MultiTileEntityContainer tTileEntityContainer = mBlock.mMultiTileEntityRegistry.getNewTileEntityContainer(aStack);
		if (tTileEntityContainer != null && tTileEntityContainer.mTileEntity instanceof IMTE_OnDespawn) try {return ((IMTE_OnDespawn)tTileEntityContainer.mTileEntity).onDespawn(aEntity, aStack);} catch(Throwable e) {e.printStackTrace(ERR);}
		return 0;
	}
	
    @Override
	public int getEntityLifespan(ItemStack aStack, World aWorld) {
    	MultiTileEntityContainer tTileEntityContainer = mBlock.mMultiTileEntityRegistry.getNewTileEntityContainer(aStack);
		if (tTileEntityContainer != null && tTileEntityContainer.mTileEntity instanceof IMTE_GetLifeSpan) try {return ((IMTE_GetLifeSpan)tTileEntityContainer.mTileEntity).getLifeSpan(aWorld, aStack);} catch(Throwable e) {e.printStackTrace(ERR);}
		return super.getEntityLifespan(aStack, aWorld);
    }
    
	@Override
	@SideOnly(Side.CLIENT)
	public void getSubItems(Item aItem, CreativeTabs aTab, List aList) {
		if (aTab instanceof CreativeTab) {
			for (MultiTileEntityClassContainer tClass : mBlock.mMultiTileEntityRegistry.mRegistrations) if (!tClass.mHidden || SHOW_HIDDEN_ITEMS) if (((CreativeTab)aTab).mMetaData == tClass.mCreativeTabID) if (!(tClass.mCanonicalTileEntity instanceof IMTE_GetSubItems) || ((IMTE_GetSubItems)tClass.mCanonicalTileEntity).getSubItems(mBlock, aItem, aTab, aList, tClass.mID)) aList.add(mBlock.mMultiTileEntityRegistry.getItem(tClass.mID));
		} else {
			for (MultiTileEntityClassContainer tClass : mBlock.mMultiTileEntityRegistry.mRegistrations) if (!tClass.mHidden || SHOW_HIDDEN_ITEMS) if (!(tClass.mCanonicalTileEntity instanceof IMTE_GetSubItems) || ((IMTE_GetSubItems)tClass.mCanonicalTileEntity).getSubItems(mBlock, aItem, aTab, aList, tClass.mID)) aList.add(mBlock.mMultiTileEntityRegistry.getItem(tClass.mID));
		}
	}
    
	@Override
    public CreativeTabs[] getCreativeTabs() {
        return mBlock.mMultiTileEntityRegistry.mCreativeTabs.values().toArray(new CreativeTabs[mBlock.mMultiTileEntityRegistry.mCreativeTabs.size()]);
    }
    
    @Override
	public boolean onItemUse(ItemStack aStack, EntityPlayer aPlayer, World aWorld, int aX, int aY, int aZ, int aSide, float aHitX, float aHitY, float aHitZ) {
        if (aY < 0 || aY > aWorld.getHeight()) return F;
        
        try {
	    	Block tClickedBlock = aWorld.getBlock(aX, aY, aZ);
	        if (tClickedBlock instanceof BlockSnow && (aWorld.getBlockMetadata(aX, aY, aZ) & 7) < 1) {
	            aSide = SIDE_TOP;
	        } else if (tClickedBlock != Blocks.vine && tClickedBlock != Blocks.tallgrass && tClickedBlock != Blocks.deadbush && !tClickedBlock.isReplaceable(aWorld, aX, aY, aZ)) {
	            aX += OFFSETS_X[aSide]; aY += OFFSETS_Y[aSide]; aZ += OFFSETS_Z[aSide];
	        }
	    	Block tReplacedBlock = aWorld.getBlock(aX, aY, aZ);
	    	
	        if (aStack.stackSize == 0 || (aPlayer != null && !aPlayer.canPlayerEdit(aX, aY, aZ, aSide, aStack))) return F;
	        
	        MultiTileEntityContainer aMTEContainer = mBlock.mMultiTileEntityRegistry.getNewTileEntityContainer(aStack);
	        
	        if (aMTEContainer != null && aWorld.canPlaceEntityOnSide(aMTEContainer.mBlock, aX, aY, aZ, F, aSide, aPlayer, aStack) && (!(aMTEContainer.mTileEntity instanceof IMTE_CanPlace) || ((IMTE_CanPlace)aMTEContainer.mTileEntity).canPlace(aStack, aPlayer, aWorld, aX, aY, aZ, (byte)aSide, aHitX, aHitY, aHitZ)) && aWorld.setBlock(aX, aY, aZ, aMTEContainer.mBlock, 15-aMTEContainer.mBlockMetaData, 2)) {
	        	((IMultiTileEntity)aMTEContainer.mTileEntity).setShouldRefresh(F);
	            UT.Worlds.setTileEntity(aWorld, aX, aY, aZ, aMTEContainer.mTileEntity, F);
	            aWorld.setBlock(aX, aY, aZ, aMTEContainer.mBlock, aMTEContainer.mBlockMetaData, 0);
	            ((IMultiTileEntity)aMTEContainer.mTileEntity).setShouldRefresh(T);
	            UT.Worlds.setTileEntity(aWorld, aX, aY, aZ, aMTEContainer.mTileEntity, T);
	            
	            if (!(aMTEContainer.mTileEntity instanceof IMTE_OnPlaced) || ((IMTE_OnPlaced)aMTEContainer.mTileEntity).onPlaced(aStack, aPlayer, aMTEContainer, aWorld, aX, aY, aZ, (byte)aSide, aHitX, aHitY, aHitZ)) {
	            	aWorld.playSoundEffect(aX+0.5, aY+0.5, aZ+0.5, aMTEContainer.mBlock.stepSound.func_150496_b(), (aMTEContainer.mBlock.stepSound.getVolume()+1) / 2, aMTEContainer.mBlock.stepSound.getPitch()*0.8F);
	            }
	            if (aMTEContainer.mTileEntity instanceof IMTE_HasMultiBlockMachineRelevantData) {
	            	if (((IMTE_HasMultiBlockMachineRelevantData)aMTEContainer.mTileEntity).hasMultiBlockMachineRelevantData()) ITileEntityMachineBlockUpdateable.Util.causeMachineUpdate(aWorld, new ChunkCoordinates(aX, aY, aZ), aMTEContainer.mBlock, aMTEContainer.mBlockMetaData, F);
	            }
	            if (!aWorld.isRemote) {
	            	aWorld.notifyBlockChange(aX, aY, aZ, tReplacedBlock);
	                aWorld.func_147453_f(aX, aY, aZ, aMTEContainer.mBlock);
	            }
	            if (aMTEContainer.mTileEntity instanceof ITileEntity) {
	            	((ITileEntity)aMTEContainer.mTileEntity).onTileEntityPlaced();
	            }
	            aWorld.func_147451_t(aX, aY, aZ);
	        	aStack.stackSize--;
	        	return T;
	        }
        } catch(Throwable e) {
        	e.printStackTrace(ERR);
        }
		return F;
    }
    
	@Override
	public void updateItemStack(ItemStack aStack) {
		//
	}
	
	@Override
	public int getItemStackLimit(ItemStack aStack) {
		MultiTileEntityClassContainer tContainer = mBlock.mMultiTileEntityRegistry.getClassContainer(aStack);
		return tContainer == null ? 1 : tContainer.mStackSize;
	}
	
	@Override
    public void onCreated(ItemStack aStack, World aWorld, EntityPlayer aPlayer) {
		MultiTileEntityContainer tTileEntityContainer = mBlock.mMultiTileEntityRegistry.getNewTileEntityContainer(aStack);
	    if (tTileEntityContainer != null && tTileEntityContainer.mTileEntity instanceof IMTE_OnCrafted) {
			updateItemStack(aStack);
	    	((IMTE_OnCrafted)tTileEntityContainer.mTileEntity).onCrafted(aPlayer, aWorld, aStack);
	    }
		updateItemStack(aStack);
    }
	
	@Override
	public OreDictItemData getOreDictItemData(ItemStack aStack) {
		ArrayListNoNulls<OreDictItemData> rList = new ArrayListNoNulls<OreDictItemData>(F, OM.data(aStack));
		MultiTileEntityContainer tTileEntityContainer = mBlock.mMultiTileEntityRegistry.getNewTileEntityContainer(aStack);
		if (tTileEntityContainer != null && tTileEntityContainer.mTileEntity instanceof ITileEntityCoverable) {
			CoverData tCoverData = ((ITileEntityCoverable)tTileEntityContainer.mTileEntity).getCoverData();
			if (tCoverData != null) for (byte tSide : ALL_SIDES_VALID) rList.add(OM.anydata(tCoverData.getCoverItem(tSide)));
		}
		if (tTileEntityContainer != null && tTileEntityContainer.mTileEntity instanceof ITileEntityFoamable && ((ITileEntityFoamable)tTileEntityContainer.mTileEntity).hasFoam(SIDE_UNKNOWN)) {
			rList.add(new OreDictItemData(MT.ConstructionFoam, U));
		}
		return rList.isEmpty()?null:new OreDictItemData(rList);
	}
	
	@Override public final String getUnlocalizedName() {return mBlock.mMultiTileEntityRegistry.mNameInternal;}
	@Override public final String getUnlocalizedName(ItemStack aStack) {return mBlock.mMultiTileEntityRegistry.mNameInternal+"."+getDamage(aStack);}
	@Override public final boolean hasContainerItem(ItemStack aStack) {return getContainerItem(aStack) != null;}
	@Override public ItemStack getContainerItem(ItemStack aStack) {return null;}
    @Override public int getSpriteNumber() {return 0;}
    @Override public void registerIcons(IIconRegister aRegister) {/**/}
	@Override public boolean isBookEnchantable(ItemStack aStack, ItemStack aBook) {return F;}
	@Override public boolean getIsRepairable(ItemStack aStack, ItemStack aMaterial) {return F;}
	@Override public int getItemEnchantability() {return 0;}
}